/*
 *  $Id: reduce.c,v 1.4 1993/11/12 12:30:51 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */


#ifndef lint
static char vcid[] = "$Id: reduce.c,v 1.4 1993/11/12 12:30:51 doss Exp $";
#endif /* lint */

#include <mpi.h>
#include "coll.h"

/*@

MPI_Reduce - 

@*/
MPI_Reduce ( sendbuf, recvbuf, count, datatype, op, root, comm )
void             *sendbuf;
void             *recvbuf;
int               count;
MPI_Datatype      datatype;
void            (*op)();
int               root;
MPI_Comm  comm;
{
  int              size, rank;
  MPI_Group        group;
  MPI_Comm_request req;
  MPI_Status       status;

  /* Check for invalid arguments */
  if ( (comm            == (MPI_Comm)0) ||
       (root            <  0) || 
       (comm->comm_type == MPIR_INTER) ) 
    return (MPI_SUCCESS); 
  
  /* Is root within the communicator? */
  MPI_Comm_size ( comm, &size );
  if ( root >= size )
    return (MPI_SUCCESS);

  /* Get my rank and switch communicators to the hidden collective */
  MPI_Comm_rank ( comm, &rank );
  comm = comm->comm_coll;

  /* If I'm the root, then I will receive and reduce the messages */
  if (rank == root) {
    int              extent, i;
    void            *buffer;

    /* Copy my buffer to the recvbuf */
    MPI_Extent ( datatype, &extent );
    buffer = (void *)MALLOC(extent * count);
    if (sendbuf != recvbuf) {
      MPI_Isend( sendbuf, count, datatype, root, MPIR_REDUCE_TAG, comm, &req );
      MPI_Wait( req, &status );
      MPI_Irecv( recvbuf, count, datatype, root, MPIR_REDUCE_TAG, comm, &req);
      MPI_Wait( req, &status );
    }

    /* Receive messages from everywhere and combine buffers */
    for (i=0; i<root; i++) {
      MPI_Irecv( buffer, count, datatype, i, 
                 MPIR_REDUCE_TAG, comm, &req);
      MPI_Wait( req, &status );
      (*op)(buffer, recvbuf, count, datatype); 
    }
    for (i=root+1; i<size; i++) {
      MPI_Irecv( buffer, count, datatype, i, 
                 MPIR_REDUCE_TAG, comm, &req);
      MPI_Wait( req, &status );
      (*op)(buffer, recvbuf, count, datatype); 
    }
  }
  else {
    MPI_Isend( sendbuf, count, datatype, root, MPIR_REDUCE_TAG, comm, &req );
    MPI_Wait( req, &status );
  }
  return (MPI_SUCCESS);
}


