/*
 *  $Id: group_rincl.c,v 1.3 1993/11/12 12:00:48 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

/*@

MPI_Group_range_incl - 

@*/

#include <mpi.h>


int MPI_Group_range_incl ( group, n, ranges, group_out )
MPI_Group group, *group_out;
int       n, *ranges;
{
  int i, j, k, ranks, first, last, stride;
  int np = 0;
  MPI_Group new_group;

  /* Check for a NULL input group or NULL sized new group */
  if ( (group == (MPI_Group)0) || (n <= 0) ) {
    (*group_out) = (MPI_Group)0;
    return (MPI_SUCCESS);
  }
  
  /* Determine the number of ranks that will be included */
  for (i=0; i<n; i++)
    if (ranges[(i*3)+2] != 0)
      if ( (ranks=((ranges[(i*3)+1]-ranges[i*3])/ranges[(i*3)+2])+1) > 0 )
        np += ranks;

  /* Check for np == 0 ranks to include */
  if ( np <=0 ) {
    (*group_out) = (MPI_Group)0;
    return (MPI_SUCCESS);
  }

  /* Create the new group */
  new_group = (*group_out)  = NEW(MPIR_GROUP);
  new_group->ref_count      = 1;
  new_group->local_rank     = MPI_UNDEFINED;
  new_group->set_mark       = (uint32 *)0;
  new_group->np             = np;
  new_group->lrank_to_grank = (uint32 *) MALLOC( np * sizeof(uint32) );

  /* Fill in the lrank_to_grank list */
  k = 0;
  for (i=0; i<n; i++) {
    first = ranges[i*3]; last = ranges[(i*3)+1]; stride = ranges[(i*3)+2];
    if (stride != 0)
      for ( j=first; j*stride <= last*stride; j += stride ) {
        if ( (j < group->np) && (j >= 0) ) {
          if ( group->local_rank == j )  
            new_group->local_rank = k;
          new_group->lrank_to_grank[k++] = group->lrank_to_grank[j];
        }
        else
          new_group->lrank_to_grank[k++] = MPI_UNDEFINED;
      }
  }

  /* Determine the previous and next powers of 2 */
  MPIR_Powers_of_2 ( new_group->np, &(new_group->N2_next), &(new_group->N2_prev) );

  return (MPI_SUCCESS);
}

