/*
 *  $Id: cart_rank.c,v 1.4 1993/11/12 12:35:46 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Cart_rank - determines process rank in communicator given cartesian
                location

@*/
int MPI_Cart_rank ( comm, coords, rank )
MPI_Comm comm;
int *coords;
int *rank;
{
  int i, ndims, multiplier = 1;

  /* Check for valid arguments */
  if ( (comm                         == (MPI_Comm)0) ||
       (rank                         == (int *)0)            || 
       (comm->topology.type          != MPI_CART)            ||
       (comm->topology.tag.cart.ndims <  1) )
    return (MPI_SUCCESS);

  /* Compute rank */
  ndims = comm->topology.tag.cart.ndims;
  (*rank) = coords[ndims-1];
  if ( ((*rank) == MPI_PROCNULL)                          ||
       ((*rank) >= comm->topology.tag.cart.dims[ndims-1]) ||
       ((*rank) <  0) ) {
    (*rank) = MPI_PROCNULL;
    return (MPI_SUCCESS);
  }
    
  for ( i=ndims-2; i >=0; i-- ) {
    if ( (coords[i] == MPI_PROCNULL)                    ||
	 (coords[i] >= comm->topology.tag.cart.dims[i]) ||
	 (coords[i] <  0) ) {
      (*rank) = MPI_PROCNULL;
      return (MPI_SUCCESS);
    }
    multiplier *= comm->topology.tag.cart.dims[i+1];
    (*rank) += multiplier * coords[i];
  }

  return (MPI_SUCCESS);
}
